SEEDS_MONGODB_USERNAME = 'SEEDS_MONGODB_USERNAME'
SEEDS_MONGODB_PASSWORD = 'SEEDS_MONGODB_PASSWORD'

SEEDS_MONGODB_HOST = 'SEEDS_MONGODB_HOST'
SEEDS_MONGODB_PORT = 'SEEDS_MONGODB_PORT'

SEEDS_MONGODB_DATABASE = 'SEEDS_MONGODB_DATABASE'
SEEDS_MONGODB_COLLECTION = 'SEEDS_MONGODB_COLLECTION'

SEEDS_MONGODB_OPTIONS_ = 'SEEDS_MONGODB_OPTIONS_'

SEEDS_MONGODB_SEEDS_QUERY = 'SEEDS_MONGODB_SEEDS_QUERY'
SEEDS_MONGODB_SEEDS_BATCH_SIZE = 'SEEDS_MONGODB_SEEDS_BATCH_SIZE'

SEEDS_MONGODB_SEEDS_PREPARE = 'SEEDS_MONGODB_SEEDS_PREPARE'
